package com.dayou.mapper;

import com.dayou.entity.MajorTarget;
import com.dayou.dao.CustomBaseMapper;
import com.dayou.vo.IdNameVO;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;
import java.util.Set;

/**
 * <p>
 * 大中型项目评估对象 Mapper 接口
 * </p>
 *
 * @author wucl
 * @since 2023-11-03
 */
public interface MajorTargetMapper extends CustomBaseMapper<MajorTarget> {

    void updateSubReportNo(@Param("list") List<IdNameVO> list,@Param("exist") String statementNo);

    void updateSubStatementNo(@Param("list") List<IdNameVO> list);

    void updateSubLetterNo(@Param("list") List<IdNameVO> list,@Param("exist") String statementNo);

    void updateMainReportNo(@Param("mainNumber") String mainNumber, @Param("businessId")Long businessId,@Param("statementNo")String statementNo);

    void updateMainStatementNo(@Param("mainNumber") String mainNumber, @Param("businessId")Long businessId);

    void updateMainLetterNo(@Param("mainNumber") String mainNumber, @Param("businessId")Long businessId, @Param("statementNo")String statementNo);

    List<MajorTarget> getSubReportNo(@Param("businessId")Long businessId);

    void cleanSubStatementNo(@Param("set") Set<String> statementNos);

    void cleanSubReportNo(@Param("set") Set<String> reportNos);

    void cleanSubLetterNo(@Param("set") Set<String> letterNos);

    List<MajorTarget> getList(@Param("vo") MajorTarget majorTarget);

    BigDecimal getEvaluateAmountByTargetId(@Param("targetIds") List<Long> targetIds);
}
